/* -*- tab-width: 4 -*-
 *
 * Electric(tm) VLSI Design System
 *
 * File: SeaOfGatesCell.java
 *
 * Copyright (c) 2011, Oracle and/or its affiliates. All rights reserved.
 *
 * Electric(tm) is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Electric(tm) is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Electric(tm); see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, Mass 02111-1307, USA.
 */
package com.sun.electric.tool.user.dialogs;

import com.sun.electric.database.EditingPreferences;
import com.sun.electric.database.hierarchy.Cell;
import com.sun.electric.technology.ArcProto;
import com.sun.electric.technology.Technology;
import com.sun.electric.tool.Job;
import com.sun.electric.tool.JobException;
import com.sun.electric.tool.routing.SeaOfGates.SeaOfGatesCellParameters;
import com.sun.electric.tool.user.User;
import com.sun.electric.tool.user.ui.EditWindow;
import com.sun.electric.tool.user.ui.TopLevel;
import com.sun.electric.util.TextUtils;

import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Iterator;

import javax.swing.DefaultListModel;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.ListSelectionModel;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

/**
 * Class to handle the "Sea-of-Gates Cell Preferences" dialog.
 */
public class SeaOfGatesCell extends EDialog
{
    SeaOfGatesCellParameters sogp;
	private JList arcsList;
	private DefaultListModel arcsModel;
	private Technology curTech;

	public static void showSeaOfGatesCellDialog()
	{
		JFrame jf = null;
		if (TopLevel.isMDIMode())
			jf = TopLevel.getCurrentJFrame();
		SeaOfGatesCell theDialog = new SeaOfGatesCell(jf);
		theDialog.setVisible(true);
	}

	/** Creates new form Sea-of-Gates Cell Preferences */
	private SeaOfGatesCell(Frame parent)
	{
		super(parent, true);
		initComponents();

		EditWindow wnd = EditWindow.needCurrent();
		if (wnd == null) return;
		Cell cell = wnd.getCell();
		if (cell == null) return;
		jLabel1.setText("Sea-of-Gates Parameters for Cell " + cell.describe(false));
		sogp = new SeaOfGatesCellParameters(cell);

		// see if steiner trees are already done
		sogNoSteinerTrees.setSelected(sogp.isSteinerDone());
		if (!sogp.isFavorHorVer()) sogNoHorVer.setSelected(true); else
		{
			if (sogp.isHorizontalEven()) sogVerOddHorEven.setSelected(true); else
				sogHorOddVerEven.setSelected(true);
		}

		sogFavorLayer.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent evt) { favorClicked(); }
		});
		sogAvoidLayer.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent evt) { preventClicked(); }
		});

		// initialize the list of arcs and technologies
		curTech = Technology.getCurrent();
		arcsModel = new DefaultListModel();
		arcsList = new JList(arcsModel);
		arcsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		arcsList.addMouseListener(new MouseAdapter()
		{
			public void mouseClicked(MouseEvent evt) { arcClicked(); }
		});
		sogArcList.setViewportView(arcsList);

		for(Iterator<Technology> tIt = Technology.getTechnologies(); tIt.hasNext(); )
		{
			Technology tech = tIt.next();
			sogTechList.addItem(tech.getTechName());
		}
		sogTechList.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent evt) { techChanged(); }
		});
		sogTechList.setSelectedItem(curTech.getTechName());

		sogHorGrid.getDocument().addDocumentListener(new SOGDocumentListener(0));
		sogHorGridOff.getDocument().addDocumentListener(new SOGDocumentListener(1));
		sogVerGrid.getDocument().addDocumentListener(new SOGDocumentListener(2));
		sogVerGridOff.getDocument().addDocumentListener(new SOGDocumentListener(3));
		pack();
	}

	/**
	 * Class to handle special changes to changes to a CIF layer.
	 */
	private class SOGDocumentListener implements DocumentListener
	{
		private int which;

		SOGDocumentListener(int which) { this.which = which; }

		public void changedUpdate(DocumentEvent e) { gridFieldChanged(which); }
		public void insertUpdate(DocumentEvent e) { gridFieldChanged(which); }
		public void removeUpdate(DocumentEvent e) { gridFieldChanged(which); }
	}

	private void favorClicked()
	{
		String str = (String)arcsList.getSelectedValue();
		ArcProto ap = curTech.findArcProto(str);
		sogp.setFavored(ap, sogFavorLayer.isSelected());
	}

	private void preventClicked()
	{
		String str = (String)arcsList.getSelectedValue();
		ArcProto ap = curTech.findArcProto(str);
		sogp.setPrevented(ap, sogAvoidLayer.isSelected());
	}

	private void gridFieldChanged(int which)
	{
		String str = (String)arcsList.getSelectedValue();
		ArcProto ap = curTech.findArcProto(str);
		switch (which)
		{
			case 0:
				double newGrid = TextUtils.atof(sogHorGrid.getText());
				sogp.setHorGrid(ap, newGrid);
				break;
			case 1:
				newGrid = TextUtils.atof(sogHorGridOff.getText());
				sogp.setHorGridOff(ap, newGrid);
				break;
			case 2:
				newGrid = TextUtils.atof(sogVerGrid.getText());
				sogp.setVerGrid(ap, newGrid);
				break;
			case 3:
				newGrid = TextUtils.atof(sogVerGridOff.getText());
				sogp.setVerGridOff(ap, newGrid);
				break;
		}
	}

	/**
	 * Method called when the user clicks on a layer name in the scrollable list.
	 */
	private void arcClicked()
	{
		String str = (String)arcsList.getSelectedValue();
		ArcProto ap = curTech.findArcProto(str);
		sogAvoidLayer.setSelected(sogp.isPrevented(ap));
		sogFavorLayer.setSelected(sogp.isFavored(ap));
		sogHorGrid.setText(TextUtils.formatDouble(sogp.getHorGrid(ap)));
		sogHorGridOff.setText(TextUtils.formatDouble(sogp.getHorGridOff(ap)));
		sogVerGrid.setText(TextUtils.formatDouble(sogp.getVerGrid(ap)));
		sogVerGridOff.setText(TextUtils.formatDouble(sogp.getVerGridOff(ap)));
	}

	/**
	 * Method called when the technology (for default arcs) has changed.
	 */
	private void techChanged()
	{
		String techName = (String)sogTechList.getSelectedItem();
		Technology tech = Technology.findTechnology(techName);
		if (tech == null) return;
		curTech = tech;

		arcsModel.clear();
		for(Iterator<ArcProto> it = tech.getArcs(); it.hasNext(); )
		{
			ArcProto ap = it.next();
			arcsModel.addElement(ap.getName());
		}
		arcsList.setSelectedIndex(0);
		arcClicked();
	}

	private void okPressed()
	{
		sogp.setSteinerDone(sogNoSteinerTrees.isSelected());
		sogp.setFavorHorVer(!sogNoHorVer.isSelected());
		if (!sogNoHorVer.isSelected())
		{
			sogp.setHorizontalEven(sogVerOddHorEven.isSelected());
		}
		new UpdateSeaOfGatesCell(sogp);
	}

	protected void escapePressed() { cancel(null); }

	/**
     * Class to delete an attribute in a new thread.
     */
    private static class UpdateSeaOfGatesCell extends Job
	{
    	private SeaOfGatesCellParameters sogp;

        private UpdateSeaOfGatesCell(SeaOfGatesCellParameters sogp)
        {
            super("Update Sea-of-Gates Cell Preferences", User.getUserTool(), Job.Type.CHANGE, null, null, Job.Priority.USER);
            this.sogp = sogp;
            startJob();
        }

        @Override
        public boolean doIt() throws JobException
        {
            EditingPreferences ep = getEditingPreferences();
            sogp.saveParameters(ep);
            return true;
        }
    }

	/** This method is called from within the constructor to
	 * initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is
	 * always regenerated by the Form Editor.
	 */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        sogHorVerChoices = new javax.swing.ButtonGroup();
        ok = new javax.swing.JButton();
        cancel = new javax.swing.JButton();
        jLabel1 = new javax.swing.JLabel();
        jPanel1 = new javax.swing.JPanel();
        sogArcList = new javax.swing.JScrollPane();
        jLabel2 = new javax.swing.JLabel();
        sogTechList = new javax.swing.JComboBox();
        jLabel3 = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        sogHorGrid = new javax.swing.JTextField();
        sogVerGrid = new javax.swing.JTextField();
        sogHorGridOff = new javax.swing.JTextField();
        sogVerGridOff = new javax.swing.JTextField();
        sogAvoidLayer = new javax.swing.JCheckBox();
        sogFavorLayer = new javax.swing.JCheckBox();
        jLabel8 = new javax.swing.JLabel();
        jLabel9 = new javax.swing.JLabel();
        jPanel2 = new javax.swing.JPanel();
        sogNoSteinerTrees = new javax.swing.JCheckBox();
        sogHorOddVerEven = new javax.swing.JRadioButton();
        sogVerOddHorEven = new javax.swing.JRadioButton();
        sogNoHorVer = new javax.swing.JRadioButton();
        jLabel7 = new javax.swing.JLabel();

        setTitle("Sea-of-Gates Cell Properties");
        setName(""); // NOI18N
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });
        getContentPane().setLayout(new java.awt.GridBagLayout());

        ok.setText("OK");
        ok.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                ok(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 4);
        getContentPane().add(ok, gridBagConstraints);

        cancel.setText("Cancel");
        cancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancel(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 4);
        getContentPane().add(cancel, gridBagConstraints);

        jLabel1.setText("Sea-of-Gates Parameters for Cell XXX");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 4);
        getContentPane().add(jLabel1, gridBagConstraints);

        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder("For Individual Layers in the Cell"));
        jPanel1.setLayout(new java.awt.GridBagLayout());

        sogArcList.setMinimumSize(new java.awt.Dimension(24, 150));
        sogArcList.setPreferredSize(new java.awt.Dimension(100, 150));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        jPanel1.add(sogArcList, gridBagConstraints);

        jLabel2.setText("Technology:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 4);
        jPanel1.add(jLabel2, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 4);
        jPanel1.add(sogTechList, gridBagConstraints);

        jLabel3.setText("Horizontal:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 2, 4);
        jPanel1.add(jLabel3, gridBagConstraints);

        jLabel4.setText("Vertical:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(2, 4, 4, 4);
        jPanel1.add(jLabel4, gridBagConstraints);

        sogHorGrid.setColumns(10);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 2, 4);
        jPanel1.add(sogHorGrid, gridBagConstraints);

        sogVerGrid.setColumns(10);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.insets = new java.awt.Insets(2, 4, 4, 4);
        jPanel1.add(sogVerGrid, gridBagConstraints);

        sogHorGridOff.setColumns(10);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 2, 4);
        jPanel1.add(sogHorGridOff, gridBagConstraints);

        sogVerGridOff.setColumns(10);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.insets = new java.awt.Insets(2, 4, 4, 4);
        jPanel1.add(sogVerGridOff, gridBagConstraints);

        sogAvoidLayer.setText("Avoid layer use");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 4);
        jPanel1.add(sogAvoidLayer, gridBagConstraints);

        sogFavorLayer.setText("Favor layer");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 4);
        jPanel1.add(sogFavorLayer, gridBagConstraints);

        jLabel8.setText("Grid Spacing:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        jPanel1.add(jLabel8, gridBagConstraints);

        jLabel9.setText("Grid Offset:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 2;
        jPanel1.add(jLabel9, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 4);
        getContentPane().add(jPanel1, gridBagConstraints);

        jPanel2.setBorder(javax.swing.BorderFactory.createTitledBorder("For Entire Cell"));
        jPanel2.setLayout(new java.awt.GridBagLayout());

        sogNoSteinerTrees.setText("Do not make Steiner Trees for nets (already done)");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 4);
        jPanel2.add(sogNoSteinerTrees, gridBagConstraints);

        sogHorVerChoices.add(sogHorOddVerEven);
        sogHorOddVerEven.setText("Horizontal: M1,3,5   Vertical: M2,4,6");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 20, 0, 4);
        jPanel2.add(sogHorOddVerEven, gridBagConstraints);

        sogHorVerChoices.add(sogVerOddHorEven);
        sogVerOddHorEven.setText("Vertical: M1,3,5   Horizontal: M2,4,6");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 20, 0, 4);
        jPanel2.add(sogVerOddHorEven, gridBagConstraints);

        sogHorVerChoices.add(sogNoHorVer);
        sogNoHorVer.setText("No Horizontal/Vertical Preference");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 20, 4, 4);
        jPanel2.add(sogNoHorVer, gridBagConstraints);

        jLabel7.setText("Horizontal/Vertical Layer Preferences:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 0, 4);
        jPanel2.add(jLabel7, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 4);
        getContentPane().add(jPanel2, gridBagConstraints);

        pack();
    }// </editor-fold>//GEN-END:initComponents

	private void ok(java.awt.event.ActionEvent evt)//GEN-FIRST:event_ok
	{//GEN-HEADEREND:event_ok
		okPressed();
		closeDialog(null);
	}//GEN-LAST:event_ok

	private void cancel(java.awt.event.ActionEvent evt)//GEN-FIRST:event_cancel
	{//GEN-HEADEREND:event_cancel
		closeDialog(null);
	}//GEN-LAST:event_cancel

	/** Closes the dialog */
	private void closeDialog(java.awt.event.WindowEvent evt)//GEN-FIRST:event_closeDialog
	{
		setVisible(false);
		dispose();

	}//GEN-LAST:event_closeDialog

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton cancel;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JButton ok;
    private javax.swing.JScrollPane sogArcList;
    private javax.swing.JCheckBox sogAvoidLayer;
    private javax.swing.JCheckBox sogFavorLayer;
    private javax.swing.JTextField sogHorGrid;
    private javax.swing.JTextField sogHorGridOff;
    private javax.swing.JRadioButton sogHorOddVerEven;
    private javax.swing.ButtonGroup sogHorVerChoices;
    private javax.swing.JRadioButton sogNoHorVer;
    private javax.swing.JCheckBox sogNoSteinerTrees;
    private javax.swing.JComboBox sogTechList;
    private javax.swing.JTextField sogVerGrid;
    private javax.swing.JTextField sogVerGridOff;
    private javax.swing.JRadioButton sogVerOddHorEven;
    // End of variables declaration//GEN-END:variables

}
